//
// Image.h: header for the Image class.
//
//
//      AUTHOR: Russell Lowke
//
//		  Date: Jan 20th 2005
//

#ifndef IMAGE_H
#define IMAGE_H

#include "tga.h"
//
// A pixel with blue, green, red, and alpha channels.
//
typedef struct {
    unsigned char b, g, r;
    unsigned char alpha;
} TGAPixel;

//
// An image stored as a rectangular array of m_pixels (otherwise known
// as a "raster" image).
class Image {

	public:
		Image();												// default constructor
		Image(int m_width, int m_height);
		Image(const Image& n);									// copy constructor
		~Image();												// destructor
		Image& operator=  (const Image& other);
		
		void fill(TGAPixel& color);
		void loadFromFile (const char *filename, bool alpha = true);
		void saveToFile   (const char *filename);
		void draw();											// draw image at default loc
		void drawAt(int x, int y);								// draw image at default loc
		
		bool clicked(int mouseX, int mouseY);					// check if mouse bounded by image rect
		void resize(int width, int height);

		// getters and setters
		inline TGAPixel& operator()(int x, int y) const 
					{ return m_pixels[m_width * y + x]; }		// get a pixel
		inline int getWidth()    const	{ return m_width; }		// get image width
		inline int getHeight()   const	{ return m_height; }	// get image height
		inline int getXLoc()     const	{ return m_xLoc; }
		inline int getYLoc()     const	{ return m_yLoc; }
		inline TGAPixel* getPixels() const	{ return m_pixels; }	// get ptr to m_pixels array

		inline void setLoc(int mX, int mY) { m_xLoc = mX;	m_yLoc = mY; }
		inline void setPixels(TGAPixel* mPixels) { m_pixels = mPixels; }
		inline void setAt(int x, int y, const TGAPixel color) { m_pixels[m_width * y + x]  = color; }

	private:

		TGAHEADER m_header;		// tga header information
		TGAPixel* m_pixels;		// array of pixel data
		int    m_xLoc;			// default x location
		int    m_yLoc;			// default y location
		int    m_width;			// width of image
		int    m_height;		// height of image

};

#endif // IMAGE_H
